# Code generated by release_script GitHub action; DO NOT EDIT MANUALLY.
ANNOTATION_UNSUPPORTED = [
    "topRoot",
    "monPol",
    "monATarget",
    "monTarget",
    "moTopProps",
    "moModifiable",
    "moOwnable",
    "moResolvable",
    "moASubj",
    "actionACont",
    "taskExec",
    "namingNamedObject",
    "namingNamedIdentifiedObject",
    "conditionInfo",
    "conditionRetP",
    "conditionRecord",
    "conditionLoggable",
    "faultInfo",
    "relnInst",
    "relnTo",
    "relnFrom",
    "statsItem",
    "statsAColl",
    "statsAThrP",
    "relnTaskRef",
    "hvsNode",
    "qosACong",
    "qosAQueue",
    "qosABuffer",
    "qosASched",
    "qosClassification",
    "qosADscpClass",
    "qosADot1PClass",
    "lacpALagPol",
    "pconsRef",
    "eventARetP",
    "faultAThrValue",
    "faultARetP",
    "configABackupP",
    "dnsAProfile",
    "dnsAProv",
    "dnsADomain",
    "dnsALbl",
    "sysdebugRepository",
    "sysdebugFile",
    "sysdebugLogBehavior",
    "firmwareSource",
    "firmwareAFwStatusCont",
    "firmwareARunning",
    "firmwareAFwP",
    "maintAMaintP",
    "maintUserNotif",
    "fabricComp",
    "fabricANode",
    "fabricALink",
    "ruleDefinition",
    "ruleItem",
    "ruleRequirement",
    "ruleSizeRequirement",
    "stpAIfPol",
    "haHaTest",
    "rtctrlAMatchRule",
    "rtctrlAMatchIpRule",
    "rtctrlAMatchRtType",
    "rtctrlASubnet",
    "rtctrlAAttrP",
    "rtctrlASetRule",
    "rtctrlASetTag",
    "rtctrlASetComm",
    "rtctrlASetRtMetric",
    "rtctrlASetPref",
    "rtctrlASetNh",
    "rtctrlASetOspfFwdAddr",
    "rtctrlASetOspfNssa",
    "bgpAAsP",
    "bgpACtxPol",
    "bgpAPeerPfxPol",
    "bgpAPeerP",
    "bgpAExtP",
    "cdpAIfPol",
    "fabricProtoIfPol",
    "fabricL2IfPol",
    "fabricL3IfPol",
    "lldpAIfPol",
    "fabricProtoPol",
    "fabricProtoComp",
    "fabricL2ProtoPol",
    "fabricL3ProtoPol",
    "fabricL2ProtoComp",
    "fabricL3ProtoComp",
    "fabricProtoInstPol",
    "fabricUtilInstPol",
    "fabricL2InstPol",
    "fabricL3InstPol",
    "fabricProtoDomPol",
    "fabricL2DomPol",
    "fabricL3DomPol",
    "fabricL3CtxPol",
    "l2AInstPol",
    "fabricMaintPol",
    "fabricNodeGrp",
    "fabricAPodBlk",
    "fabricANodeBlk",
    "fabricAPortBlk",
    "fabricSelector",
    "fabricANodeS",
    "fabricNodeS",
    "fabricACardS",
    "fabricCardS",
    "fabricAPortS",
    "fabricPortS",
    "fabricIntfPol",
    "fabricAProfile",
    "fabricProfile",
    "fabricPolGrp",
    "fabricNodeP",
    "fabricCardP",
    "fabricPortP",
    "fabricAPortPGrp",
    "fabricANodePGrp",
    "fabricACardPGrp",
    "fabricSpAPortPGrp",
    "fabricLeAPortPGrp",
    "fabricAProtPol",
    "fabricANodePEp",
    "fabricPol",
    "fabricInfrP",
    "fabricInfrExP",
    "fabricDom",
    "fabricDef",
    "fabricAPolGrp",
    "vsvcAProvLbl",
    "vsvcAConsLbl",
    "compASvcVM",
    "compNic",
    "compPNic",
    "compEntity",
    "compElement",
    "compContE",
    "compObj",
    "compCont",
    "compProvP",
    "compDomP",
    "compCtrlrP",
    "compAccessP",
    "compUsrAccP",
    "compHost",
    "compPHost",
    "vzACollection",
    "vzACtrct",
    "vzABrCP",
    "vzAIf",
    "vzAFilterable",
    "vzAFilterableUnit",
    "vzASubj",
    "vzATerm",
    "vzASTerm",
    "vzALbl",
    "vzACompLbl",
    "vzAnyToCollection",
    "vzAFilter",
    "vzAFiltEntry",
    "ospfACtxPol",
    "ospfAIfP",
    "ospfAExtP",
    "dhcpARelayP",
    "dhcpALbl",
    "dhcpAInfraProvP",
    "poolElement",
    "poolPoolable",
    "poolPoolMember",
    "poolPool",
    "pkiItem",
    "aaaDefinition",
    "commDefinition",
    "pkiDefinition",
    "aaaSystemUser",
    "aaaBanner",
    "aaaUserAction",
    "aaaARetP",
    "commComp",
    "commWeb",
    "commShell",
    "aaaRealm",
    "aaaConfig",
    "aaaAuthMethod",
    "aaaEp",
    "aaaAProvider",
    "aaaProviderGroup",
    "adcomATsInfoUnit",
    "adcomARwi",
    "healthARetP",
    "healthAInst",
    "igmpASnoopPol",
    "sysfileEp",
    "sysfileRepository",
    "sysfileInstance",
    "fileARemoteHost",
    "fileARemotePath",
    "monProtoP",
    "monSecAuthP",
    "monGroup",
    "monSubj",
    "monSrc",
    "conditionCondP",
    "ctrlrDom",
    "l2extADomP",
    "l2extALNodeP",
    "l2extAIfP",
    "l2extAInstPSubnet",
    "extnwEPg",
    "extnwOut",
    "extnwDomP",
    "extnwAInstPSubnet",
    "extnwALNodeP",
    "extnwALIfP",
    "l3extADomP",
    "l3extALNodeP",
    "l3extAIfP",
    "l3extAMember",
    "l3extAInstPSubnet",
    "trigSchedWindowP",
    "trigInst",
    "trigWindow",
    "trigSchedWindow",
    "trigExecutable",
    "trigTriggerable",
    "trigSingleTriggerable",
    "fvACont",
    "fvADeplCont",
    "fvL2Dom",
    "fvABD",
    "fvABDPol",
    "fvAEpRetPol",
    "fvComp",
    "fvATg",
    "fvEPgToCollection",
    "fvADomP",
    "fvEPg",
    "fvCEPg",
    "fvAREpPCtrct",
    "fvDom",
    "fvL3Dom",
    "fvACtx",
    "fvNwEp",
    "fvATp",
    "fvEp",
    "fvPEp",
    "fvAEpDef",
    "fvTo",
    "fvFrom",
    "mgmtAZone",
    "mgmtAInstPSubnet",
    "eqptdiagpTestSet",
    "eqptdiagpTestSetBoot",
    "eqptdiagpTestSetHealth",
    "eqptdiagpTestSetOd",
    "eqptdiagpPortTestSetOd",
    "eqptdiagpPortTestSetBt",
    "eqptdiagpPortTestSetHl",
    "eqptdiagpLpTsOd",
    "eqptdiagpFpTsOd",
    "eqptdiagpCardTestSetOd",
    "eqptdiagpSupCTsOd",
    "eqptdiagpSysCTsOd",
    "eqptdiagpFcTsOd",
    "eqptdiagpLcTsOd",
    "eqptdiagpExtChCardTsOd",
    "eqptdiagpPol",
    "eqptdiagpHealthPol",
    "eqptdiagpASynthObj",
    "oamExec",
    "pingAExec",
    "tracerouteAExec",
    "bgpAf",
    "dhcpAddr",
    "dhcpNode",
    "dhcpResp",
    "eqptALPort",
    "dbgACRulePCommon",
    "dbgacTenantSpaceCmn",
    "dbgacEpgCmn",
    "dbgacFromEpgCmn",
    "dbgacToEpgCmn",
    "dbgacFromEpCmn",
    "dbgacToEpCmn",
    "dbgexpExportP",
    "dbgexpNodeStatus",
    "spanASrcGrp",
    "spanASrc",
    "spanASpanLbl",
    "spanADest",
    "spanAVSrc",
    "spanAVSrcGrp",
    "spanAVDestGrp",
    "spanAVDest",
    "svccorePol",
    "svccoreACore",
    "traceroutepTrP",
    "syntheticObject",
    "syntheticAContext",
    "syntheticATestObj",
    "syntheticTLTestObj",
    "syntheticCTestObj",
    "monExportP",
    "ipARouteP",
    "ipANexthopP",
    "infraFexBlk",
    "infraANodeS",
    "infraNodeGrp",
    "infraPortS",
    "infraEPg",
    "infraACEPg",
    "infraAPEPg",
    "infraANode",
    "infraAIpP",
    "infraProfile",
    "infraPolGrp",
    "infraAccBaseGrp",
    "infraFexGrp",
    "infraAccGrp",
    "infraLbl",
    "infraAPEp",
    "infraACEp",
    "infraAFunc",
    "infraGeNode",
    "infraADomP",
    "infraDomP",
    "infraExP",
    "datetimeAPol",
    "datetimeANtpAuthKey",
    "datetimeANtpProv",
    "fvnsAAddrBlk",
    "fvnsAEncapBlk",
    "fvnsAInstP",
    "fvnsAVxlanInstP",
    "fvnsAAddrInstP",
    "polAttTgt",
    "snmpAPol",
    "snmpACommunityP",
    "snmpAUserP",
    "snmpAClientGrpP",
    "snmpAClientP",
    "snmpACtxP",
    "fvDef",
    "fvNp",
    "fvUp",
    "polNToRef",
    "polNFromRef",
    "polObj",
    "polDef",
    "polDefRoot",
    "polComp",
    "polInstr",
    "polCont",
    "polDom",
    "polCtrlr",
    "polCompl",
    "polComplElem",
    "polConsElem",
    "polLbl",
    "polProvLbl",
    "polConsLbl",
    "polIf",
    "polProvIf",
    "polConsIf",
    "polRelnHolder",
    "polNs",
    "polAConfIssues",
    "vnsAGraph",
    "vnsANode",
    "vnsAFuncNode",
    "vnsAFolder",
    "vnsACCfgRel",
    "vnsAParam",
    "vnsATerm",
    "vnsATermNode",
    "vnsAbsTermNode",
    "vnsALDevCtx",
    "vnsALIfCtx",
    "vnsAConn",
    "vnsAFuncConn",
    "vnsATermConn",
    "vnsAConnection",
    "vnsAL4L7ServiceFault",
    "vnsACCfg",
    "vnsALDevIf",
    "vnsALDev",
    "vnsALIf",
    "vnsALDevLIf",
    "vnsDevItem",
    "fabricAPathIssues",
    "conditionSevAsnP",
    "fvAStCEp",
    "fabricNodeToPolicy",
    "aaaADomainRef",
    "fvAEpTaskAggr",
    "vzToRFltP",
    "fvAToBD",
    "frmwrkARelDelCont",
    "infraDomainToNs",
    "infraToAInstP",
    "aaaARbacRule",
    "statsDebugItem",
    "vzInterfaceToCollection",
    "spanAToCEp",
    "fabricAOOSReln",
    "l4AVxlanInstPol",
    "fabricVxlanInstPol",
    "fabricL1IfPol",
    "frmwrkARelDelControl",
    "spanACEpDef",
    "eptrkEpRslt",
    "mcpAIfPol",
    "ipmcsnoopRtrIf",
    "bfdAf",
    "bgpALocalAsnP",
    "bgpACtxAfPol",
    "l3extADefaultRouteLeakP",
    "fvAStIp",
    "rtctrlASetRtMetricType",
    "vnsAEPpInfo",
    "ndAIfPol",
    "ndAPfxPol",
    "eigrpACtxAfPol",
    "eigrpAStubP",
    "eigrpAIfP",
    "eigrpAExtP",
    "l3extAIp",
    "fabricProtoConsFrom",
    "fabricProtoConsTo",
    "fabricAPathS",
    "infraAAccBndlGrp",
    "fabricNodeToPathOverridePolicy",
    "adcomARwiAdvanced",
    "fvAClassifier",
    "fvACrtrn",
    "fvAttr",
    "fvAVmAttr",
    "fvAIpAttr",
    "fvAMacAttr",
    "fvAProtoAttr",
    "polADependentOn",
    "vnsAVRoutingNetworks",
    "l3extARouteTagPol",
    "nwsAFwPol",
    "fabricL4IfPol",
    "fvAVip",
    "fvAAREpPUpdate",
    "conditionSummary",
    "polAPrToPol",
    "polAObjToPolReln",
    "fvACrRule",
    "fvASCrtrn",
    "fvnsAVlanInstP",
    "vzAnyToInterface",
    "fvEPgToInterface",
    "fabricPodGrp",
    "fabricAPodS",
    "infraPodGrp",
    "vnsAMgmt",
    "fabricPolicyGrpToMonitoring",
    "plannerIPs",
    "plannerAObject",
    "plannerAEpg",
    "compAPltfmP",
    "compAVmmPltfmP",
    "compAVmmSecP",
    "bgpARRP",
    "bgpARtTargetP",
    "bgpARtTarget",
    "l3extAFabricExtRoutingP",
    "fvAFabricExtConnP",
    "fvAPodConnP",
    "fvAPeeringP",
    "fvANode",
    "plannerATmpl",
    "nwsASrc",
    "nwsASyslogSrc",
    "throttlerASub",
    "qosADscpTrans",
    "fvARsToRemote",
    "vnsOrchReq",
    "vnsOrchResp",
    "fcprARs",
    "dbgexpTechSupOnDBase",
    "infraGeSnNode",
    "l3extAConsLbl",
    "l3extAProvLbl",
    "trigATriggeredWindow",
    "qosADppPol",
    "bfdAInstPol",
    "bfdAIfPol",
    "bfdAIfP",
    "plannerADomainTmpl",
    "plannerAEpgDomain",
    "vmmACapObj",
    "vmmACapInfo",
    "fvARsToRemoteFC",
    "rtctrlAMatchCommFactor",
    "rtctrlAMatchCommTerm",
    "rtctrlAMatchCommRegexTerm",
    "rtctrlASetDamp",
    "rtctrlASetWeight",
    "hvsContE",
    "hvsUsegContE",
    "compAPvlanP",
    "usegAUsegEPg",
    "fvAStAttr",
    "fvADyAttr",
    "mgmtAIp",
    "faultARsToRemote",
    "analyticsACfgSrv",
    "infraAAccGrp",
    "infraSpAccGrp",
    "infraANodeP",
    "infraPortP",
    "infraAPathS",
    "ipmcsnoopMcSrc",
    "ipmcsnoopTgtIf",
    "analyticsACluster",
    "rtdmcAIfPol",
    "rtdmcARtMapPol",
    "rtdmcAExtP",
    "rtdmcACtxPol",
    "rtdmcARPPol",
    "rtdmcAAutoRPPol",
    "rtdmcABSRPPol",
    "rtdmcAStaticRPPol",
    "rtdmcAStaticRPEntry",
    "rtdmcARPGrpRangePol",
    "rtdmcARegTrPol",
    "rtdmcAResPol",
    "rtdmcAPatPol",
    "rtdmcAASMPatPol",
    "rtdmcASGRangeExpPol",
    "rtdmcASharedRangePol",
    "rtdmcASSMPatPol",
    "rtdmcASSMRangePol",
    "rtdmcABidirPatPol",
    "pimAIfP",
    "rtdmcARtMapEntry",
    "rtdmcAJPFilterPol",
    "rtdmcANbrFilterPol",
    "rtdmcAMAFilter",
    "rtdmcABSRFilter",
    "rtdmcAFilterPol",
    "fvAEPgPathAtt",
    "fvnsAVsanInstP",
    "ipmcAIfPol",
    "ipmcAStRepPol",
    "ipmcAStateLPol",
    "ipmcARepPol",
    "ipmcACtxPol",
    "ipmcASSMXlateP",
    "igmpAIfP",
    "vmmEpgAggr",
    "apPluginName",
    "rtdmcAIfPolCont",
    "fabricL2PortSecurityPol",
    "fcAPinningP",
    "fcAPinningLbl",
    "fabricAONodeS",
    "infraAONodeS",
    "analyticsRemoteNode",
    "analyticsTarget",
    "fabricASubPortBlk",
    "bgpARtTargetInstrP",
    "hsrpAIfPol",
    "hsrpAGroupPol",
    "hsrpAGroupP",
    "hsrpASecVip",
    "hsrpAIfP",
    "netflowARecordPol",
    "netflowAExporterPol",
    "netflowARsInterfaceToMonitor",
    "netflowARsMonitorToRecord",
    "netflowARsMonitorToExporter",
    "netflowAMonitorPol",
    "igmpASnoopStaticGroup",
    "igmpASnoopAccessGroup",
    "netflowAFabExporterPol",
    "fabricQinqIfPol",
    "orchsEntity",
    "orchsElement",
    "vnsNATReq",
    "vnsLBReq",
    "aclACL",
    "aclL3ACE",
    "apDockerName",
    "hostprotASubj",
    "macsecAAIfPol",
    "macsecAIfPol",
    "macsecAAParamPol",
    "macsecAParamPol",
    "macsecAAKeyChainPol",
    "macsecAAKeyPol",
    "macsecAKeyChainPol",
    "macsecAKeyPol",
    "fvAIntersiteConnP",
    "fvAIntersiteConnPDef",
    "fvASiteConnP",
    "coppAProfile",
    "coppACustomValues",
    "infraNodeS",
    "rtctrlASetASPath",
    "rtctrlASetASPathASN",
    "fvADnsAttr",
    "dnsepgFault",
    "dnsepgASvrGrp",
    "dnsepgAMgmt",
    "dnsepgASvr",
    "dnsepgADomain",
    "ipANexthopEpP",
    "vmmInjectedObject",
    "poeAIfPol",
    "iaclAProfile",
    "telemetryAServer",
    "snmpATrapFwdServerP",
    "dwdmAOptChnlIfPol",
    "vzARuleOwner",
    "telemetryAServerPol",
    "telemetryARemoteServer",
    "telemetryAFlowServers",
    "telemetryAServerP",
    "plannerAEpgFilter",
    "callhomeASrc",
    "callhomeADest",
    "callhomeAGroup",
    "fvAEpAnycast",
    "tagTag",
    "tagAnnotation",
    "tagASelector",
    "infraFcAccGrp",
    "infraAFcAccBndlGrp",
    "vsanARsVsanPathAtt",
    "vsanARtVsanPathAtt",
    "vzSubjectToFilter",
    "telemetryAStreamEnable",
    "cloudsecASaKeyP",
    "rtdmcAInterVRFPol",
    "rtdmcAInterVRFEntry",
    "cloudsecAControl",
    "adepgAResElement",
    "adepgAElement",
    "adepgACont",
    "adepgAOrgUnit",
    "adepgEntity",
    "adepgContE",
    "fvAIdAttr",
    "edmObj",
    "edmCont",
    "edmGroupP",
    "edmMgrP",
    "edmEntity",
    "edmElement",
    "edmContE",
    "authASvrGroup",
    "authASvr",
    "authBaseUsrAccP",
    "cloudsecASaKeyPLocal",
    "cloudsecASaKeyPRemote",
    "cloudsecASaKeyStatus",
    "cloudsecASaKeyStatusLocal",
    "cloudsecASaKeyStatusRemote",
    "cloudsecASpKeySt",
    "cloudsecACapability",
    "aaaRbacAnnotation",
    "lacpAEnhancedLagPol",
    "compAHvHealth",
    "mldASnoopPol",
    "fvAExtRoutes",
    "fvAExtRoutableRemoteSitePodSubnet",
    "fvAEpNlb",
    "extdevSDWanASlaPol",
    "eigrpAAuthIfP",
    "fvAAKeyChainPol",
    "fvAAKeyPol",
    "fvAKeyChainPol",
    "fvAKeyPol",
    "fvSyntheticIp",
    "edmACapFlags",
    "cloudABgpPeerP",
    "cloudABgpAsP",
    "cloudARouterP",
    "cloudAIntNetworkP",
    "cloudAExtNetworkP",
    "cloudAHostRouterPol",
    "cloudAHostBootstrapPol",
    "cloudAVpnGwPol",
    "cloudAHostIfP",
    "cloudALoopbackIfP",
    "cloudAL3IfP",
    "cloudAIpv4AddrP",
    "cloudAL3TunnelIfP",
    "cloudAIpSecTunnelIfP",
    "cloudAOspfIfP",
    "cloudAOspfAreaP",
    "cloudACtxProfile",
    "cloudACidr",
    "cloudASubnet",
    "cloudAAwsLogGroup",
    "cloudAAwsFlowLogPol",
    "cloudAProvider",
    "cloudAAwsProvider",
    "cloudAAEPg",
    "cloudAEPSelector",
    "cloudAEPSelectorDef",
    "cloudADomP",
    "genericsARule",
    "ipsecAIsakmpPhase1Pol",
    "ipsecAIsakmpPhase2Pol",
    "resolutionARsToRemote",
    "cloudtemplateASubnetPool",
    "fvAACrtrn",
    "fvAUsegAssocBD",
    "cloudALDev",
    "cloudAListener",
    "cloudAPool",
    "cloudAListenerRule",
    "cloudARuleCondition",
    "cloudARuleAction",
    "cloudACertStore",
    "cloudACertificate",
    "arpAIfPol",
    "compNameIdentEntity",
    "edmAOperCont",
    "edmAStatsCont",
    "cloudASvcPol",
    "hcloudATgStats",
    "statsANWStatsObj",
    "statsATunnel",
    "cloudAController",
    "cloudAApicSubnet",
    "cloudAApicSubnetPool",
    "cloudABaseEPg",
    "cloudASvcEPg",
    "statsAALbStats",
    "mldASnoopStaticGroup",
    "mldASnoopAccessGroup",
    "datetimeANtpIFFKey",
    "hcloudRouterStateOper",
    "vmmAUplinkP",
    "vmmAUplinkPCont",
    "cloudAAFilter",
    "fvASDWanPrefixTaskAggr",
    "cloudAProvResP",
    "bfdAMhInstPol",
    "bfdAMhIfPol",
    "fvAEPSelector",
    "ptpAACfg",
    "ptpACfg",
    "fabricL2BDPol",
    "rtdmcABDPol",
    "rtdmcABDFilter",
    "ptpAProfile",
    "mdpADomP",
    "mdpADomainPeeringPol",
    "mdpAPeeringDomain",
    "mdpANodeP",
    "mdpAClassId",
    "mdpATenant",
    "mplsAExtP",
    "mplsAIfP",
    "mplsALabelPol",
    "mplsANodeSidP",
    "qosAMplsIngressRule",
    "qosMplsMarking",
    "qosAMplsEgressRule",
    "cloudAGatewayRouterP",
    "cloudATransitHubGwPol",
    "cloudAL3IntTunnelIfP",
    "cloudABdiId",
    "mplsASrgbLabelPol",
    "bfdAMhNodePol",
    "bfdANodeP",
    "leakASubnet",
    "rtctrlASetNhUnchanged",
    "mdpAService",
    "cloudALIf",
    "cloudAParamPol",
    "cloudAComputePol",
    "cloudAMgmtPol",
    "cloudANWParams",
    "fvRtScopeFrom",
    "cloudACtxUnderlayP",
    "cloudAHealthProbe",
    "leakARouteCont",
    "netflowAVmmExporterPol",
    "cloudABrownfield",
    "cloudAMapping",
    "cloudASelectedEP",
    "statsANlbStats",
    "mdpAEntity",
    "dbgANodeInst",
    "snmpACtrlrInst",
    "cloudAFrontendIPv4Addr",
    "qosAUburst",
    "telemetryAFteEvents",
    "telemetryAFteEventsTcpFlags",
    "telemetryAFteEventsExt",
    "cloudACloudSvcEPg",
    "hcloudASvcResBase",
    "bgpADomainIdBase",
    "fvAEpTag",
    "bgpASiteOfOriginP",
    "fvAEPSelectorDef",
    "vmmCFaultInfo",
    "synceAAIfPol",
    "synceAIfPol",
    "rtctrlASetRedistMultipath",
    "cloudANextHopIp",
    "cloudAVrfRouteLeakPol",
    "hcloudALeakedPfx",
    "leakAPrefix",
    "fvARogueExceptionMac",
    "bfdAMicroBfdP",
    "hcloudAIntPfx",
    "hcloudAExtPfx",
    "cloudAVpnNetworkP",
    "snmpUser",
    "cloudAVirtualWanP",
    "fvAFBRGroup",
    "fvAFBRoute",
    "fvAFBRMember",
    "rtctrlASetPolicyTag",
    "fvACtxRtSummPol",
    "fvARtSummSubnet",
    "fvAIntraVrfFabricImpRtctrlP",
    "fvAFabricExpRtctrlP",
    "cloudAVip",
    "rtctrlAMatchAsPathRegexTerm",
    "cloudAGcpFlowLogPol",
    "statsAGcpNWStatsObj",
    "cloudABfdPol",
    "cloudABfdP",
    "bgpACtxAddlPathPol",
    "xcvrOpticsIfPol",
    "xcvrOpticsFabIfPol",
    "hostprotANamespace",
    "fvARouteDeployP",
    "rtdmcACSWPol",
    "rtdmcACSWEntry",
    "analyticsTo",
    "analyticsFrom",
    "dmemotltestAbsObject",
    "bgpAsnmpBgpTraps",
    "l3extARogueExceptionMac",
    "l3extARogueExceptionMacGroup",
    "vzAFiltPZEntry",
]
